@@ -31,7 +31,7 @@
 #include "gr-cooking-view.h"
 #include "gr-recipe.h"
 #include "gr-recipe-formatter.h"
-#include "gr-images.h"
+#include "gr-image.h"
 #include "gr-utils.h"
 #include "gr-timer.h"
 #include "gr-window.h"
@@ -50,7 +50,7 @@ struct _GrCookingView
         GtkWidget *text_box;
         GtkWidget *timer_box;
 
-        GArray *images;
+        GPtrArray *images;
         char *id;
         char *instructions;
 
@@ -64,6 +64,8 @@ struct _GrCookingView
 #endif
 
         GList *timers;
+
+        GCancellable *cancellable;
 };
 
 typedef struct
@@ -139,12 +141,12 @@ timer_active (GrTimer    *timer,
 }
 
 static StepData *
-step_data_new (int         num,
-               int         n_steps,
-               const char *text,
-               guint64     duration,
-               const char *title,
-               int         image,
+step_data_new (int            num,
+               int            n_steps,
+               const char    *text,
+               guint64        duration,
+               const char    *title,
+               int            image,
                GrCookingView *view)
 {
         StepData *d;
@@ -240,8 +242,11 @@ gr_cooking_view_finalize (GObject *object)
 {
         GrCookingView *self = GR_COOKING_VIEW (object);
 
+        g_cancellable_cancel (self->cancellable);
+        g_clear_object (&self->cancellable);
+
         g_clear_pointer (&self->id, g_free);
-        g_clear_pointer (&self->images, g_array_unref);
+        g_clear_pointer (&self->images, g_ptr_array_unref);
         g_clear_pointer (&self->instructions, g_free);
         g_clear_pointer (&self->steps, g_ptr_array_unref);
 
@@ -272,6 +277,14 @@ gr_cooking_view_init (GrCookingView *self)
 #endif
 }
 
+static void
+set_pixbuf (GrImage   *ri,
+            GdkPixbuf *pixbuf,
+            gpointer   data)
+{
+        gtk_image_set_from_pixbuf (GTK_IMAGE (data), pixbuf);
+}
+
 static void
 setup_step (GrCookingView *view)
 {
@@ -280,6 +293,9 @@ setup_step (GrCookingView *view)
         if (!view->images)
                 return;
 
+        g_cancellable_cancel (view->cancellable);
+        g_clear_object (&view->cancellable);
+
         s = g_ptr_array_index (view->steps, view->step);
 
         if (s->heading && s->heading[0]) {
@@ -318,14 +334,18 @@ setup_step (GrCookingView *view)
                 GrImage *ri = NULL;
                 g_autoptr(GdkPixbuf) pixbuf = NULL;
 
+                view->cancellable = g_cancellable_new ();
+
                 gtk_widget_show (view->cooking_stack);
                 gtk_widget_set_halign (view->text_box, GTK_ALIGN_START);
-                ri = &g_array_index (view->images, GrImage, s->image);
-                if (view->wide)
-                        pixbuf = load_pixbuf_fill_size (ri->path, 640, 480);
-                else
-                        pixbuf = load_pixbuf_fill_size (ri->path, 320, 240);
-                gtk_image_set_from_pixbuf (GTK_IMAGE (view->cooking_image), pixbuf);
+                ri = g_ptr_array_index (view->images, s->image);
+                gr_image_load (ri,
+                               view->wide ? 640 : 320,
+                               view->wide ? 480 : 240,
+                               FALSE,
+                               view->cancellable,
+                               set_pixbuf,
+                               view->cooking_image);
                 gtk_stack_set_visible_child_name (GTK_STACK (view->cooking_stack), "image");
         }
         else {
@@ -584,14 +604,14 @@ void
 gr_cooking_view_set_data (GrCookingView *view,
                           const char    *id,
                           const char    *instructions,
-                          GArray        *images)
+                          GPtrArray     *images)
 {
         g_free (view->id);
         view->id = g_strdup (id);
         g_free (view->instructions);
         view->instructions = g_strdup (instructions);
-        g_clear_pointer (&view->images, g_array_unref);
-        view->images = g_array_ref (images);
+        g_clear_pointer (&view->images, g_ptr_array_unref);
+        view->images = g_ptr_array_ref (images);
 
         setup_steps (view);
 }
@@ -630,10 +650,9 @@ gr_cooking_view_start (GrCookingView *view)
 }
 
 void
-gr_cooking_view_stop (GrCookingView *view)
+gr_cooking_view_stop (GrCookingView *view,
+                      gboolean       stop_timers)
 {
-        GList *l;
-
         g_object_set (view->cooking_timer, "timer", NULL, NULL);
         if (view->timer_box)
                 container_remove_all (GTK_CONTAINER (view->timer_box));
@@ -642,10 +661,18 @@ gr_cooking_view_stop (GrCookingView *view)
         g_clear_pointer (&view->images, g_array_unref);
         g_ptr_array_set_size (view->steps, 0);
 
-        for (l = view->timers; l; l = l->next) {
-                GrTimer *timer = l->data;
-                TimerData *td = g_object_get_data (G_OBJECT (timer), "timer-data");
-                td->use_system = TRUE;
+        if (stop_timers) {
+                g_list_foreach (view->timers, (GFunc)gr_timer_reset, NULL);
+                g_list_free_full (view->timers, g_object_unref);
+                view->timers = NULL;
+        }
+        else {
+                GList *l;
+
+                for (l = view->timers; l; l = l->next) {
+                        TimerData *td = g_object_get_data (G_OBJECT (l->data), "timer-data");
+                        td->use_system = TRUE;
+                }
         }
 }
 
