@@ -31,10 +31,9 @@
 
 #include "gr-details-page.h"
 #include "gr-recipe-store.h"
-#include "gr-app.h"
 #include "gr-window.h"
 #include "gr-utils.h"
-#include "gr-images.h"
+#include "gr-image.h"
 #include "gr-image-viewer.h"
 #include "gr-ingredients-list.h"
 #include "gr-timer.h"
@@ -57,6 +56,7 @@ struct _GrDetailsPage
         GrRecipePrinter *printer;
         GrRecipeExporter *exporter;
 
+        GtkWidget *main_content;
         GtkWidget *recipe_image;
         GtkWidget *prep_time_desc;
         GtkWidget *prep_time_label;
@@ -106,7 +106,7 @@ delete_recipe (GrDetailsPage *page)
 
         recipe = g_object_ref (page->recipe);
 
-        store = gr_app_get_recipe_store (GR_APP (g_application_get_default ()));
+        store = gr_recipe_store_get ();
         gr_recipe_store_remove_recipe (store, page->recipe);
         g_set_object (&page->recipe, NULL);
         g_set_object (&page->chef, NULL);
@@ -196,7 +196,7 @@ cook_it_later (GrDetailsPage *page)
 {
         GrRecipeStore *store;
 
-        store = gr_app_get_recipe_store (GR_APP (g_application_get_default ()));
+        store = gr_recipe_store_get ();
         if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (page->favorite_button)))
                 gr_recipe_store_add_favorite (store, page->recipe);
         else
@@ -210,7 +210,7 @@ shop_it (GrDetailsPage *page)
         GtkWidget *window;
         int serves;
 
-        store = gr_app_get_recipe_store (GR_APP (g_application_get_default ()));
+        store = gr_recipe_store_get ();
         serves = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (page->serves_spin));
         gr_recipe_store_add_to_shopping (store, page->recipe, serves);
 
@@ -266,9 +266,9 @@ save_notes (gpointer data)
 
         g_object_set (page->recipe, "notes", text, NULL);
 
-        store = gr_app_get_recipe_store (GR_APP (g_application_get_default ()));
+        store = gr_recipe_store_get ();
         if (!gr_recipe_store_update_recipe (store, page->recipe, id, &error)) {
-                g_message ("Error: %s", error->message);
+                g_warning ("Error: %s", error->message);
         }
 
 out:
@@ -308,7 +308,7 @@ activate_uri_at_idle (gpointer data)
                 const char *id;
                 g_autoptr(GrRecipe) recipe = NULL;
 
-                store = gr_app_get_recipe_store (GR_APP (g_application_get_default ()));
+                store = gr_recipe_store_get ();
 
                 id = uri + strlen ("recipe:");
                 recipe = gr_recipe_store_get_recipe (store, id);
@@ -346,7 +346,7 @@ static void
 activate_image (GrDetailsPage *page)
 {
         GtkWidget *window;
-        g_autoptr(GArray) images = NULL;
+        g_autoptr(GPtrArray) images = NULL;
         int idx;
 
         g_object_get (page->recipe_image, "images", &images, "index", &idx, NULL);
@@ -393,6 +393,7 @@ gr_details_page_class_init (GrDetailsPageClass *klass)
 
         gtk_widget_class_set_template_from_resource (widget_class, "/org/gnome/Recipes/gr-details-page.ui");
 
+        gtk_widget_class_bind_template_child (widget_class, GrDetailsPage, main_content);
         gtk_widget_class_bind_template_child (widget_class, GrDetailsPage, recipe_image);
         gtk_widget_class_bind_template_child (widget_class, GrDetailsPage, prep_time_desc);
         gtk_widget_class_bind_template_child (widget_class, GrDetailsPage, prep_time_label);
@@ -562,6 +563,7 @@ process_instructions (const char *instructions)
 
         for (i = 0; i < steps->len; i++) {
                 GrRecipeStep *step = (GrRecipeStep *)g_ptr_array_index (steps, i);
+                g_autofree char *escaped = NULL;
 
                 if (i > 0)
                         g_string_append (s, "\n\n");
@@ -591,12 +593,22 @@ process_instructions (const char *instructions)
                         g_string_append (s, "\">♦</a>");
                 }
 
-                g_string_append (s, step->text);
+                escaped = g_markup_escape_text (step->text, -1);
+                g_string_append (s, escaped);
         }
 
         return g_string_free (s, FALSE);
 }
 
+static void
+scroll_up (GrDetailsPage *page)
+{
+        GtkAdjustment *adj;
+
+        adj = gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW (page->main_content));
+        gtk_adjustment_set_value (adj, gtk_adjustment_get_lower (adj));
+}
+
 void
 gr_details_page_set_recipe (GrDetailsPage *page,
                             GrRecipe      *recipe)
@@ -618,7 +630,7 @@ gr_details_page_set_recipe (GrDetailsPage *page,
         GrRecipeStore *store;
         g_autoptr(GrChef) chef = NULL;
         g_autoptr(GrIngredientsList) ing = NULL;
-        g_autoptr(GArray) images = NULL;
+        GPtrArray *images;
         gboolean favorite;
         gboolean same_recipe;
         int index;
@@ -647,7 +659,7 @@ gr_details_page_set_recipe (GrDetailsPage *page,
         else
                 index = gr_recipe_get_default_image (recipe);
 
-        g_object_get (recipe, "images", &images, NULL);
+        images = gr_recipe_get_images (recipe);
         gr_image_viewer_set_images (GR_IMAGE_VIEWER (page->recipe_image), images, index);
 
         ing = gr_ingredients_list_new (ingredients);
@@ -717,7 +729,7 @@ gr_details_page_set_recipe (GrDetailsPage *page,
         gtk_spin_button_set_value (GTK_SPIN_BUTTON (page->serves_spin), want_serves);
         gtk_widget_set_sensitive (page->serves_spin, ing != NULL);
 
-        store = gr_app_get_recipe_store (GR_APP (g_application_get_default ()));
+        store = gr_recipe_store_get ();
 
         favorite = gr_recipe_store_is_favorite (store, recipe);
         gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (page->favorite_button), favorite);
@@ -768,6 +780,8 @@ gr_details_page_set_recipe (GrDetailsPage *page,
                 gtk_widget_show (page->edit_button);
                 gtk_widget_show (page->delete_button);
         }
+
+        scroll_up (page);
 }
 
 GrRecipe *
@@ -800,7 +814,7 @@ connect_store_signals (GrDetailsPage *page)
 {
         GrRecipeStore *store;
 
-        store = gr_app_get_recipe_store (GR_APP (g_application_get_default ()));
+        store = gr_recipe_store_get ();
 
         g_signal_connect_swapped (store, "recipe-changed", G_CALLBACK (details_page_reload), page);
 }
