@@ -38,7 +38,7 @@
 #include "gr-cuisine.h"
 #include "gr-meal.h"
 #include "gr-season.h"
-#include "gr-images.h"
+#include "gr-image.h"
 #include "gr-image-viewer.h"
 #include "gr-ingredient-row.h"
 #include "gr-ingredient.h"
@@ -138,6 +138,8 @@ struct _GrEditPage
 
         char *author;
         gboolean unsaved;
+
+        GCancellable *cancellable;
 };
 
 G_DEFINE_TYPE (GrEditPage, gr_edit_page, GTK_TYPE_BOX)
@@ -167,24 +169,29 @@ static void
 populate_image_flowbox (GrEditPage *page)
 {
         int i;
-        g_autoptr(GArray) images = NULL;
+        GPtrArray *images;
         GtkWidget *button;
 
-        g_object_get (page->images, "images", &images, NULL);
+        g_cancellable_cancel (page->cancellable);
+        g_clear_object (&page->cancellable);
+        page->cancellable = g_cancellable_new ();
+
+        images = gr_image_viewer_get_images (GR_IMAGE_VIEWER (page->images));
 
         container_remove_all (GTK_CONTAINER (page->image_flowbox));
 
         for (i = 0; i < images->len; i++) {
-                GrImage *ri = &g_array_index (images, GrImage, i);
-                g_autoptr(GdkPixbuf) pb = load_pixbuf_fill_size (ri->path, 60, 40);
+                GrImage *ri = g_ptr_array_index (images, i);
                 GtkWidget *image;
                 GtkWidget *child;
 
-                image = gtk_image_new_from_pixbuf (pb);
+                image = gtk_image_new ();
                 gtk_widget_show (image);
                 gtk_container_add (GTK_CONTAINER (page->image_flowbox), image);
                 child = gtk_widget_get_parent (image);
                 g_object_set_data (G_OBJECT (child), "image-idx", GINT_TO_POINTER (i));
+
+                gr_image_load (ri, 60, 40, FALSE, page->cancellable, gr_image_set_pixbuf, image);
         }
 
         button = gtk_button_new ();
@@ -197,10 +204,10 @@ populate_image_flowbox (GrEditPage *page)
 static void
 update_image_button_sensitivity (GrEditPage *page)
 {
-        g_autoptr(GArray) images = NULL;
+        GPtrArray *images;
         int length;
 
-        g_object_get (page->images, "images", &images, NULL);
+        images = gr_image_viewer_get_images (GR_IMAGE_VIEWER (page->images));
         length = images->len;
         gtk_widget_set_sensitive (page->add_image_button, TRUE);
         gtk_widget_set_sensitive (page->remove_image_button, length > 0);
@@ -346,6 +353,9 @@ edit_page_finalize (GObject *object)
 {
         GrEditPage *self = GR_EDIT_PAGE (object);
 
+        g_cancellable_cancel (self->cancellable);
+        g_clear_object (&self->cancellable);
+
         if (self->index_handler_id)
                 g_signal_handler_disconnect (self->recipe, self->index_handler_id);
         g_clear_object (&self->recipe);
@@ -366,17 +376,26 @@ edit_page_finalize (GObject *object)
 static void
 populate_cuisine_combo (GrEditPage *page)
 {
-        const char **names;
-        const char *title;
-        int length;
+        g_autofree char **names = NULL;
+        guint length;
         int i;
+        GrRecipeStore *store;
 
-        names = gr_cuisine_get_names (&length);
-        for (i = 0; i < length; i++) {
-                gr_cuisine_get_data (names[i], &title, NULL, NULL);
-                gtk_combo_box_text_append (GTK_COMBO_BOX_TEXT (page->cuisine_combo),
-                                           names[i],
-                                           title);
+        store = gr_recipe_store_get ();
+        names = gr_recipe_store_get_all_cuisines (store, &length);
+        gtk_combo_box_text_remove_all (GTK_COMBO_BOX_TEXT (page->cuisine_combo));
+
+        for (i = 0; i < length; ++i) {
+                g_autofree char *temp_cuisine_name = NULL;
+
+                temp_cuisine_name = g_strstrip (g_strdup (names[i]));
+                if (strcmp (temp_cuisine_name, "") != 0) {
+                        const char *title;
+
+                        gr_cuisine_get_data (temp_cuisine_name, &title, NULL, NULL);
+                        gtk_combo_box_text_append (GTK_COMBO_BOX_TEXT (page->cuisine_combo),
+                                                                       temp_cuisine_name, title);
+                }
         }
 }
 
@@ -572,6 +591,62 @@ update_ingredient_row (GtkWidget  *row,
         g_object_set_data_full (G_OBJECT (row), "ingredient", g_strdup (ingredient), g_free);
 }
 
+static void
+up_ingredient (GtkButton  *button,
+               GrEditPage *page)
+{
+        GtkWidget *list;
+        GtkWidget *row;
+        gint selected_index;
+
+        row = gtk_widget_get_ancestor (GTK_WIDGET (button), GTK_TYPE_LIST_BOX_ROW);
+        if (!row)
+                return;
+
+        list = gtk_widget_get_parent (row);
+
+        if (row == page->active_row) {
+                page->active_row = NULL;
+                selected_index = gtk_list_box_row_get_index (GTK_LIST_BOX_ROW (row));
+                g_object_ref (row);
+                gtk_container_remove (GTK_CONTAINER (list), GTK_WIDGET (row));
+                gtk_list_box_insert (GTK_LIST_BOX (list), GTK_WIDGET (row), selected_index - 1);
+                g_object_unref (row);
+                gtk_list_box_unselect_all (GTK_LIST_BOX (list));
+                gtk_list_box_select_row (GTK_LIST_BOX (list), GTK_LIST_BOX_ROW (row));
+        }
+
+        set_unsaved (page);
+}
+
+static void
+down_ingredient (GtkButton  *button,
+                 GrEditPage *page)
+{
+        GtkWidget *list;
+        GtkWidget *row;
+        gint selected_index;
+
+        row = gtk_widget_get_ancestor (GTK_WIDGET (button), GTK_TYPE_LIST_BOX_ROW);
+        if (!row)
+                return;
+
+        list = gtk_widget_get_parent (row);
+
+        if (row == page->active_row) {
+                page->active_row = NULL;
+                selected_index = gtk_list_box_row_get_index (GTK_LIST_BOX_ROW (row));
+                g_object_ref (row);
+                gtk_container_remove (GTK_CONTAINER (list), GTK_WIDGET (row));
+                gtk_list_box_insert (GTK_LIST_BOX (list), GTK_WIDGET (row), selected_index + 1);
+                g_object_unref (row);
+                gtk_list_box_unselect_all (GTK_LIST_BOX (list));
+                gtk_list_box_select_row (GTK_LIST_BOX (list), GTK_LIST_BOX_ROW (row));
+        }
+
+        set_unsaved (page);
+}
+
 static GtkWidget *
 add_ingredient_row (GrEditPage   *page,
                     GtkWidget    *list,
@@ -619,6 +694,22 @@ add_ingredient_row (GrEditPage   *page,
         g_object_set (buttons, "margin", 4, NULL);
         button = gtk_button_new ();
         gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
+        g_signal_connect (button, "clicked", G_CALLBACK (up_ingredient), page);
+        image = gtk_image_new_from_icon_name ("go-up-symbolic", 1);
+        gtk_container_add (GTK_CONTAINER (button), image);
+        gtk_style_context_add_class (gtk_widget_get_style_context (button), "image-button");
+        gtk_style_context_add_class (gtk_widget_get_style_context (button), "circular");
+        gtk_container_add (GTK_CONTAINER (buttons), button);
+        button = gtk_button_new ();
+        gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
+        g_signal_connect (button, "clicked", G_CALLBACK (down_ingredient), page);
+        image = gtk_image_new_from_icon_name ("go-down-symbolic", 1);
+        gtk_container_add (GTK_CONTAINER (button), image);
+        gtk_style_context_add_class (gtk_widget_get_style_context (button), "image-button");
+        gtk_style_context_add_class (gtk_widget_get_style_context (button), "circular");
+        gtk_container_add (GTK_CONTAINER (buttons), button);
+        button = gtk_button_new ();
+        gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
         g_signal_connect (button, "clicked", G_CALLBACK (edit_ingredient), page);
         image = gtk_image_new_from_icon_name ("document-edit-symbolic", 1);
         gtk_container_add (GTK_CONTAINER (button), image);
@@ -1391,7 +1482,7 @@ preview_visible_changed (GrEditPage *page)
                 gr_cooking_view_stop (GR_COOKING_VIEW (page->cooking_view), TRUE);
         }
         else {
-                g_autoptr(GArray) images = NULL;
+                GPtrArray *images;
                 g_autofree char *instructions = NULL;
 
                 gtk_widget_set_sensitive (page->add_step_button, FALSE);
@@ -1401,7 +1492,7 @@ preview_visible_changed (GrEditPage *page)
                 gtk_widget_set_visible (page->prev_step_button, TRUE);
                 gtk_widget_set_visible (page->next_step_button, TRUE);
 
-                g_object_get (page->images, "images", &images, NULL);
+                images = gr_image_viewer_get_images (GR_IMAGE_VIEWER (page->images));
                 instructions = get_text_view_text (GTK_TEXT_VIEW (page->instructions_field));
 
                 gr_cooking_view_set_data (GR_COOKING_VIEW (page->cooking_view), NULL, instructions, images);
@@ -1906,7 +1997,7 @@ scroll_up (GrEditPage *page)
 void
 gr_edit_page_clear (GrEditPage *page)
 {
-        GArray *images;
+        g_autoptr(GPtrArray) images = NULL;
         GrRecipeStore *store;
 
         gr_image_viewer_revert_changes (GR_IMAGE_VIEWER (page->images));
@@ -1934,7 +2025,6 @@ gr_edit_page_clear (GrEditPage *page)
 
         images = gr_image_array_new ();
         g_object_set (page->images, "images", images, NULL);
-        g_array_unref (images);
 
         if (page->index_handler_id) {
                 g_signal_handler_disconnect (page->recipe, page->index_handler_id);
@@ -2059,7 +2149,7 @@ gr_edit_page_edit (GrEditPage *page,
         const char *ingredients;
         GrDiets diets;
         int index;
-        g_autoptr(GArray) images = NULL;
+        GPtrArray *images;
         g_autoptr(GrChef) chef = NULL;
         GrRecipeStore *store;
 
@@ -2081,8 +2171,7 @@ gr_edit_page_edit (GrEditPage *page,
         ingredients = gr_recipe_get_ingredients (recipe);
         author = gr_recipe_get_author (recipe);
         index = gr_recipe_get_default_image (recipe);
-
-        g_object_get (recipe, "images", &images, NULL);
+        images = gr_recipe_get_images (recipe);
 
         g_free (page->author);
         page->author = g_strdup (author);
@@ -2150,7 +2239,7 @@ gr_edit_page_save (GrEditPage *page)
         g_autoptr(GError) error = NULL;
         gboolean ret = TRUE;
         GrDiets diets;
-        g_autoptr(GArray) images = NULL;
+        GPtrArray *images;
 
         store = gr_recipe_store_get ();
 
@@ -2171,7 +2260,7 @@ gr_edit_page_save (GrEditPage *page)
                (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (page->vegetarian_check)) ? GR_DIET_VEGETARIAN : 0) |
                 (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (page->milk_free_check)) ? GR_DIET_MILK_FREE : 0);
 
-        g_object_get (page->images, "images", &images, NULL);
+        images = gr_image_viewer_get_images (GR_IMAGE_VIEWER (page->images));
 
         if (page->recipe) {
                 g_autofree char *old_id = NULL;
@@ -2230,9 +2319,10 @@ gr_edit_page_save (GrEditPage *page)
                                        "images", images,
                                        NULL);
                 ret = gr_recipe_store_add_recipe (store, recipe, &error);
-                g_set_object (&page->recipe, recipe);
+                if (ret)
+                        g_set_object (&page->recipe, recipe);
         }
-
+        populate_cuisine_combo (page);
         if (ret) {
                 gr_image_viewer_persist_changes (GR_IMAGE_VIEWER (page->images));
 
