@@ -27,7 +27,7 @@
 #include <gio/gunixoutputstream.h>
 
 #include "gr-image-viewer.h"
-#include "gr-images.h"
+#include "gr-image.h"
 #include "gr-utils.h"
 #include "gr-window.h"
 
@@ -70,7 +70,7 @@ struct _GrImageViewer
         GtkWidget *preview_revealer;
         GtkWidget *preview_list;
 
-        GArray *images;
+        GPtrArray *images;
         int index;
 
         GPtrArray *additions;
@@ -79,6 +79,9 @@ struct _GrImageViewer
         guint hide_timeout;
 
         GtkGesture *gesture;
+
+        GCancellable *cancellable;
+        GCancellable *preview_cancellable;
 };
 
 
@@ -113,6 +116,12 @@ gr_image_viewer_finalize (GObject *object)
 {
         GrImageViewer *viewer = GR_IMAGE_VIEWER (object);
 
+        g_cancellable_cancel (viewer->cancellable);
+        g_clear_object (&viewer->cancellable);
+
+        g_cancellable_cancel (viewer->preview_cancellable);
+        g_clear_object (&viewer->preview_cancellable);
+
         gr_image_viewer_revert_changes (viewer);
 
         g_clear_pointer (&viewer->additions, g_ptr_array_unref);
@@ -130,6 +139,9 @@ set_current_image (GrImageViewer *viewer)
 {
         GtkFlowBoxChild *child;
 
+        g_cancellable_cancel (viewer->cancellable);
+        g_clear_object (&viewer->cancellable);
+
         if (viewer->index >= viewer->images->len) {
                 gtk_stack_set_visible_child_name (GTK_STACK (viewer->stack), "placeholder");
                 return;
@@ -140,16 +152,17 @@ set_current_image (GrImageViewer *viewer)
                 g_autoptr(GdkPixbuf) pixbuf = NULL;
                 const char *vis;
 
-                ri = &g_array_index (viewer->images, GrImage, viewer->index);
-                pixbuf = load_pixbuf_fill_size (ri->path, 360, 240);
+                viewer->cancellable = g_cancellable_new ();
+
+                ri = g_ptr_array_index (viewer->images, viewer->index);
 
                 vis = gtk_stack_get_visible_child_name (GTK_STACK (viewer->stack));
                 if (strcmp (vis, "image1") == 0) {
-                        gtk_image_set_from_pixbuf (GTK_IMAGE (viewer->image2), pixbuf);
+                        gr_image_load (ri, 360, 240, FALSE, viewer->cancellable, gr_image_set_pixbuf, viewer->image2);
                         gtk_stack_set_visible_child_name (GTK_STACK (viewer->stack), "image2");
                 }
                 else {
-                        gtk_image_set_from_pixbuf (GTK_IMAGE (viewer->image1), pixbuf);
+                        gr_image_load (ri, 360, 240, FALSE, viewer->cancellable, gr_image_set_pixbuf, viewer->image1);
                         gtk_stack_set_visible_child_name (GTK_STACK (viewer->stack), "image1");
                 }
         }
@@ -166,16 +179,21 @@ populate_preview (GrImageViewer *viewer)
 {
         int i;
 
+        g_cancellable_cancel (viewer->preview_cancellable);
+        g_clear_object (&viewer->preview_cancellable);
+        viewer->preview_cancellable = g_cancellable_new ();
+
         container_remove_all (GTK_CONTAINER (viewer->preview_list));
 
         for (i = 0; i < viewer->images->len; i++) {
-                GrImage *ri = &g_array_index (viewer->images, GrImage, i);
-                g_autoptr(GdkPixbuf) pb = load_pixbuf_fill_size (ri->path, 60, 40);
+                GrImage *ri = g_ptr_array_index (viewer->images, i);
                 GtkWidget *image;
 
-                image = gtk_image_new_from_pixbuf (pb);
+                image = gtk_image_new ();
                 gtk_widget_show (image);
                 gtk_container_add (GTK_CONTAINER (viewer->preview_list), image);
+
+                gr_image_load (ri, 60, 40, FALSE, viewer->preview_cancellable, gr_image_set_pixbuf, image);
         }
 }
 
@@ -346,12 +364,10 @@ preview_selected (GrImageViewer *viewer)
 
 static void
 add_image (GrImageViewer  *viewer,
-           GrImage *ri,
+           GrImage        *ri,
            gboolean        select)
 {
-        g_array_append_vals (viewer->images, ri, 1);
-        ri = &g_array_index (viewer->images, GrImage, viewer->images->len - 1);
-        ri->path = g_strdup (ri->path);
+        g_ptr_array_add (viewer->images, ri);
 
         populate_preview (viewer);
         if (select)
@@ -369,7 +385,7 @@ image_received (GtkClipboard *clipboard,
         GrImageViewer *viewer = data;
 
         if (pixbuf) {
-                GrImage ri;
+                GrImage *ri;
                 g_autofree char *dir = NULL;
                 g_autofree char *path = NULL;
                 int fd;
@@ -388,9 +404,8 @@ image_received (GtkClipboard *clipboard,
                         return;
                 }
 
-                ri.path = g_strdup (path);
-
-                add_image (viewer, &ri, TRUE);
+                ri = gr_image_new (gr_app_get_soup_session (GR_APP (g_application_get_default ())), "local", path);
+                add_image (viewer, ri, TRUE);
         }
 }
 
@@ -485,7 +500,7 @@ gr_image_viewer_set_property (GObject      *object,
           {
           case PROP_IMAGES:
                   gr_image_viewer_set_images (self,
-                                              (GArray *) g_value_get_boxed (value),
+                                              (GPtrArray *) g_value_get_boxed (value),
                                               gr_image_viewer_get_index (self));
                   break;
 
@@ -544,24 +559,16 @@ gr_image_viewer_class_init (GrImageViewerClass *klass)
 
 void
 gr_image_viewer_set_images (GrImageViewer *viewer,
-                            GArray        *images,
+                            GPtrArray     *images,
                             int            index)
 {
-        int i;
-
         g_object_freeze_notify (G_OBJECT (viewer));
 
-        g_array_remove_range (viewer->images, 0, viewer->images->len);
+        g_ptr_array_unref (viewer->images);
+        viewer->images = g_ptr_array_ref (images);
+
         g_object_notify (G_OBJECT (viewer), "images");
 
-        for (i = 0; i < images->len; i++) {
-                GrImage *ri = &g_array_index (images, GrImage, i);
-
-                g_array_append_vals (viewer->images, ri, 1);
-                ri = &g_array_index (viewer->images, GrImage, viewer->images->len - 1);
-                ri->path = g_strdup (ri->path);
-        }
-
         populate_preview (viewer);
         viewer->index = index;
         set_current_image (viewer);
@@ -570,6 +577,12 @@ gr_image_viewer_set_images (GrImageViewer *viewer,
         g_object_thaw_notify (G_OBJECT (viewer));
 }
 
+GPtrArray *
+gr_image_viewer_get_images (GrImageViewer *viewer)
+{
+        return viewer->images;
+}
+
 static void
 file_chooser_response (GtkNativeDialog *self,
                        gint             response_id,
@@ -580,15 +593,14 @@ file_chooser_response (GtkNativeDialog *self,
 
                 names = gtk_file_chooser_get_filenames (GTK_FILE_CHOOSER (self));
                 for (l = names; l; l = l->next) {
-                        GrImage ri;
+                        GrImage *ri;
+                        g_autofree char *path = NULL;
 
-                        ri.path = import_image (l->data);
+                        path = import_image (l->data);
+                        ri = gr_image_new (gr_app_get_soup_session (GR_APP (g_application_get_default ())), "local", path);
+                        add_image (viewer, ri, TRUE);
 
-                        add_image (viewer, &ri, TRUE);
-
-                        g_ptr_array_add (viewer->additions, g_strdup (ri.path));
-
-                        g_free (ri.path);
+                        g_ptr_array_add (viewer->additions, g_strdup (path));
                 }
                 g_slist_free_full (names, g_free);
 
@@ -639,11 +651,11 @@ gr_image_viewer_remove_image (GrImageViewer *viewer)
 {
         GrImage *ri;
 
-        ri = &g_array_index (viewer->images, GrImage, viewer->index);
+        ri = g_ptr_array_index (viewer->images, viewer->index);
 
-        g_ptr_array_add (viewer->removals, g_strdup (ri->path));
+        g_ptr_array_add (viewer->removals, g_strdup (gr_image_get_path (ri)));
 
-        g_array_remove_index (viewer->images, viewer->index);
+        g_ptr_array_remove_index (viewer->images, viewer->index);
 
         if (viewer->index < viewer->images->len) {
                 populate_preview (viewer);
@@ -674,15 +686,11 @@ gr_image_viewer_rotate_image (GrImageViewer *viewer,
 
         g_assert (angle == 0 || angle == 90 || angle == 180 || angle == 270);
 
-        ri = &g_array_index (viewer->images, GrImage, viewer->index);
-
-        path = rotate_image (ri->path, angle);
-        g_ptr_array_add (viewer->removals, g_strdup (ri->path));
-
-        g_free (ri->path);
-        ri->path = path;
-
-        g_ptr_array_add (viewer->additions, g_strdup (ri->path));
+        ri = g_ptr_array_index (viewer->images, viewer->index);
+        path = rotate_image (gr_image_get_path (ri), angle);
+        gr_image_set_path (ri, path);
+        g_ptr_array_add (viewer->removals, g_strdup (path));
+        g_ptr_array_add (viewer->additions, g_strdup (path));
 
         populate_preview (viewer);
         set_current_image (viewer);
