@@ -25,10 +25,9 @@
 
 #include "gr-recipe-formatter.h"
 #include "gr-ingredients-list.h"
-#include "gr-images.h"
+#include "gr-image.h"
 #include "gr-chef.h"
 #include "gr-recipe-store.h"
-#include "gr-app.h"
 #include "gr-utils.h"
 
 char *
@@ -44,7 +43,7 @@ gr_recipe_format (GrRecipe *recipe)
         int length;
         g_autoptr(GPtrArray) steps = NULL;
 
-        store = gr_app_get_recipe_store (GR_APP (g_application_get_default ()));
+        store = gr_recipe_store_get ();
         chef = gr_recipe_store_get_chef (store, gr_recipe_get_author (recipe));
 
         s = g_string_new ("");
