@@ -25,7 +25,7 @@
 #include "gr-recipe-printer.h"
 #include "gr-recipe-formatter.h"
 #include "gr-ingredients-list.h"
-#include "gr-images.h"
+#include "gr-image.h"
 #include "gr-chef.h"
 #include "gr-recipe-store.h"
 #include "gr-utils.h"
@@ -108,8 +108,7 @@ begin_print (GtkPrintOperation *operation,
         double page_height;
         GList *page_breaks;
         g_autoptr(GString) s = NULL;
-        g_autoptr(GArray) images = NULL;
-        GrImage *ri;
+        GPtrArray *images;
         PangoRectangle title_rect;
         PangoRectangle left_rect;
         int num_lines;
@@ -128,11 +127,11 @@ begin_print (GtkPrintOperation *operation,
         width = gtk_print_context_get_width (context);
         height = gtk_print_context_get_height (context);
 
-        g_object_get (printer->recipe, "images", &images, NULL);
+        images = gr_recipe_get_images (printer->recipe);
         if (images && images->len > 0) {
-                int def_index = gr_recipe_get_default_image(printer->recipe);
-                ri = &g_array_index (images, GrImage, def_index);
-                printer->image = load_pixbuf_fit_size (ri->path, width / 2, height / 4, FALSE);
+                int def_index = gr_recipe_get_default_image (printer->recipe);
+                GrImage *ri = g_ptr_array_index (images, def_index);
+                printer->image = gr_image_load_sync (ri, width / 2, height / 4, TRUE);
         }
 
         title_font = pango_font_description_from_string ("Cantarell Bold 18");
@@ -431,7 +430,7 @@ gr_recipe_printer_print (GrRecipePrinter *printer,
 {
         GtkPrintOperation *operation;
 
-        if (in_flatpak_sandbox () && !portal_available (printer->window, "org.freedesktop.portal.Printing"))
+        if (in_flatpak_sandbox () && !portal_available (printer->window, "org.freedesktop.portal.Print"))
                 return;
 
         printer->recipe = g_object_ref (recipe);
