@@ -24,12 +24,15 @@
 #include <glib.h>
 #include <glib/gi18n.h>
 #include <glib/gstdio.h>
+#include <libsoup/soup.h>
 
 #include "gr-recipe-store.h"
 #include "gr-recipe.h"
+#include "gr-settings.h"
 #include "gr-utils.h"
 #include "gr-ingredients-list.h"
-#include "gr-images.h"
+#include "gr-image.h"
+#include "gr-app.h"
 
 
 /**
@@ -145,6 +148,9 @@ struct _GrRecipeStore
 
         GDateTime *favorite_change;
         GDateTime *shopping_change;
+
+        SoupSession *session;
+        SoupMessage *recipes_message;
 };
 
 
@@ -166,6 +172,8 @@ gr_recipe_store_finalize (GObject *object)
         g_variant_dict_unref (self->shopping_list);
         g_strfreev (self->featured_chefs);
         g_free (self->user);
+        g_clear_object (&self->recipes_message);
+        g_clear_object (&self->session);
 
         G_OBJECT_CLASS (gr_recipe_store_parent_class)->finalize (object);
 }
@@ -181,6 +189,7 @@ load_recipes (GrRecipeStore *self,
         g_auto(GStrv) groups = NULL;
         gsize length, length2;
         int i, j;
+        int version;
 
         keyfile = g_key_file_new ();
 
@@ -193,8 +202,26 @@ load_recipes (GrRecipeStore *self,
                         g_info ("No recipe db at: %s", path);
                 return FALSE;
         }
+
         g_info ("Load recipe db: %s", path);
 
+        version = g_key_file_get_integer (keyfile, "Metadata", "Version", &error);
+        if (error) {
+                if (g_error_matches (error, G_KEY_FILE_ERROR, G_KEY_FILE_ERROR_KEY_NOT_FOUND) ||
+                    g_error_matches (error, G_KEY_FILE_ERROR, G_KEY_FILE_ERROR_GROUP_NOT_FOUND)) {
+                        g_info ("No metadata found, assuming version 1");
+                        version = 1;
+                }
+                else {
+                        g_error ("Failed to read recipe db: %s", error->message);
+                        return FALSE;
+                }
+                g_clear_error (&error);
+        }
+        if (version != 1) {
+                g_error ("Don't know how to handle recipe db version %d", version);
+        }
+
         groups = g_key_file_get_groups (keyfile, &length);
         for (i = 0; i < length; i++) {
                 GrRecipe *recipe;
@@ -215,12 +242,14 @@ load_recipes (GrRecipeStore *self,
                 int spiciness;
                 int default_image = 0;
                 GrDiets diets;
-                g_autoptr(GArray) images = NULL;
-                GrImage ri;
+                g_autoptr(GPtrArray) images = NULL;
                 g_autoptr(GDateTime) ctime = NULL;
                 g_autoptr(GDateTime) mtime = NULL;
                 char *tmp;
 
+                if (strcmp (groups[i], "Metadata") == 0)
+                        continue;
+
                 g_clear_error (&error);
 
                 id = groups[i];
@@ -355,21 +384,13 @@ load_recipes (GrRecipeStore *self,
                         g_clear_error (&error);
                 }
 
-                if (paths) {
-                        for (j = 0; paths[j]; j++) {
-                                if (paths[j][0] != '/') {
-                                        tmp = g_build_filename (dir, paths[j], NULL);
-                                        g_free (paths[j]);
-                                        paths[j] = tmp;
-                                }
-                        }
-                }
-
                 images = gr_image_array_new ();
                 if (paths) {
                         for (j = 0; paths[j]; j++) {
-                                ri.path = g_strdup (paths[j]);
-                                g_array_append_val (images, ri);
+                                GrImage *ri;
+                                ri = gr_image_new (gr_app_get_soup_session (GR_APP (g_application_get_default ())), id, paths[j]);
+
+                                g_ptr_array_add (images, ri);
                         }
                 }
 
@@ -487,14 +508,13 @@ save_recipes (GrRecipeStore *self)
 
         keyfile = g_key_file_new ();
 
-        if (g_getenv ("RECIPES_DATA_DIR"))
-                dir = g_getenv ("RECIPES_DATA_DIR");
-        else
-                dir = get_user_data_dir ();
+        dir = get_user_data_dir ();
         path = g_build_filename (dir, "recipes.db", NULL);
 
         g_info ("Save recipe db: %s", path);
 
+        g_key_file_set_integer (keyfile, "Metadata", "Version", 1);
+
         keys = g_hash_table_get_keys (self->recipes);
         keys = g_list_sort (keys, (GCompareFunc)strcmp);
         for (l = keys; l; l = l->next) {
@@ -509,7 +529,7 @@ save_recipes (GrRecipeStore *self)
                 const char *ingredients;
                 const char *instructions;
                 const char *notes;
-                g_autoptr(GArray) images = NULL;
+                g_autoptr(GPtrArray) images = NULL;
                 int serves;
                 int spiciness;
                 GrDiets diets;
@@ -546,11 +566,9 @@ save_recipes (GrRecipeStore *self)
 
                 paths = g_new0 (char *, images->len + 1);
                 for (i = 0; i < images->len; i++) {
-                        GrImage *ri = &g_array_index (images, GrImage, i);
-                        if (g_str_has_prefix (ri->path, dir))
-                                paths[i] = g_strdup (ri->path + strlen (dir) + 1);
-                        else
-                                paths[i] = g_strdup (ri->path);
+                        GrImage *ri = g_ptr_array_index (images, i);
+                        const char *img_path = gr_image_get_path (ri);
+                        paths[i] = g_strdup (img_path);
                 }
 
                 // For readonly recipes, we just store notes
@@ -644,7 +662,7 @@ load_picks (GrRecipeStore *self,
 static void
 load_favorites (GrRecipeStore *self)
 {
-        g_autoptr(GSettings) settings = g_settings_new ("org.gnome.Recipes");
+        GSettings *settings = gr_settings_get ();
         gint64 timestamp;
 
         self->favorites = g_settings_get_strv (settings, "favorites");
@@ -655,7 +673,7 @@ load_favorites (GrRecipeStore *self)
 static void
 save_favorites (GrRecipeStore *self)
 {
-        g_autoptr(GSettings) settings = g_settings_new ("org.gnome.Recipes");
+        GSettings *settings = gr_settings_get ();
         gint64 timestamp;
 
         g_settings_set_strv (settings, "favorites", (const char * const *)self->favorites);
@@ -666,7 +684,7 @@ save_favorites (GrRecipeStore *self)
 static void
 load_export_list (GrRecipeStore *self)
 {
-        g_autoptr(GSettings) settings = g_settings_new ("org.gnome.Recipes");
+        GSettings *settings = gr_settings_get ();
 
         self->export_list = g_settings_get_strv (settings, "export-list");
 }
@@ -674,7 +692,7 @@ load_export_list (GrRecipeStore *self)
 static void
 save_export_list (GrRecipeStore *self)
 {
-        g_autoptr(GSettings) settings = g_settings_new ("org.gnome.Recipes");
+        GSettings *settings = gr_settings_get ();
 
         g_settings_set_strv (settings, "export-list", (const char * const *)self->export_list);
 }
@@ -682,7 +700,7 @@ save_export_list (GrRecipeStore *self)
 static void
 load_shopping (GrRecipeStore *self)
 {
-        g_autoptr(GSettings) settings = g_settings_new ("org.gnome.Recipes");
+        GSettings *settings = gr_settings_get ();
         g_autoptr(GVariant) value = NULL;
         gint64 timestamp;
 
@@ -696,7 +714,7 @@ load_shopping (GrRecipeStore *self)
 static void
 save_shopping (GrRecipeStore *self)
 {
-        g_autoptr(GSettings) settings = g_settings_new ("org.gnome.Recipes");
+        GSettings *settings = gr_settings_get ();
         g_autoptr(GVariant) value = NULL;
         gint64 timestamp;
 
@@ -720,6 +738,7 @@ load_chefs (GrRecipeStore *self,
         g_auto(GStrv) groups = NULL;
         gsize length;
         int i;
+        int version;
 
         keyfile = g_key_file_new ();
 
@@ -740,6 +759,23 @@ load_chefs (GrRecipeStore *self,
 
         g_info ("Load chefs db: %s", path);
 
+        version = g_key_file_get_integer (keyfile, "Metadata", "Version", &error);
+        if (error) {
+                if (g_error_matches (error, G_KEY_FILE_ERROR, G_KEY_FILE_ERROR_KEY_NOT_FOUND) ||
+                    g_error_matches (error, G_KEY_FILE_ERROR, G_KEY_FILE_ERROR_GROUP_NOT_FOUND)) {
+                        g_info ("No metadata found, assuming version 1");
+                        version = 1;
+                }
+                else {
+                        g_error ("Failed to read chefs db: %s", error->message);
+                        return FALSE;
+                }
+                g_clear_error (&error);
+        }
+        if (version != 1) {
+                g_error ("Don't know how to handle chefs db version %d", version);
+        }
+
         groups = g_key_file_get_groups (keyfile, &length);
         for (i = 0; i < length; i++) {
                 GrChef *chef;
@@ -749,6 +785,9 @@ load_chefs (GrRecipeStore *self,
                 g_autofree char *description = NULL;
                 g_autofree char *image_path = NULL;
 
+                if (strcmp (groups[i], "Metadata") == 0)
+                        continue;
+
                 g_clear_error (&error);
 
                 id = groups[i];
@@ -783,13 +822,6 @@ load_chefs (GrRecipeStore *self,
                         g_clear_error (&error);
                 }
 
-                if (image_path && image_path[0] != '\0' && image_path[0] != '/') {
-                        char *tmp;
-                        tmp = g_build_filename (dir, image_path, NULL);
-                        g_free (image_path);
-                        image_path = tmp;
-                }
-
                 chef = g_hash_table_lookup (self->chefs, id);
                 if (chef == NULL) {
                         chef = gr_chef_new ();
@@ -822,14 +854,13 @@ save_chefs (GrRecipeStore *store)
 
         keyfile = g_key_file_new ();
 
-        if (g_getenv ("RECIPES_DATA_DIR"))
-                dir = g_getenv ("RECIPES_DATA_DIR");
-        else
-                dir = get_user_data_dir ();
+        dir = get_user_data_dir ();
         path = g_build_filename (dir, "chefs.db", NULL);
 
         g_info ("Save chefs db: %s", path);
 
+        g_key_file_set_integer (keyfile, "Metadata", "Version", 1);
+
         keys = g_hash_table_get_keys (store->chefs);
         keys = g_list_sort (keys, (GCompareFunc)strcmp);
         for (l = keys; l; l = l->next) {
@@ -849,15 +880,7 @@ save_chefs (GrRecipeStore *store)
                 description = gr_chef_get_description (chef);
                 image_path = gr_chef_get_image (chef);
 
-                if (image_path && g_str_has_prefix (image_path, dir)) {
-                        g_autofree char *tmp2 = NULL;
-
-                        tmp2 = g_strdup (image_path + strlen (dir) + 1);
-                        g_key_file_set_string (keyfile, key, "Image", tmp2);
-                }
-                else
-                        g_key_file_set_string (keyfile, key, "Image", image_path ? image_path : "");
-
+                g_key_file_set_string (keyfile, key, "Image", image_path ? image_path : "");
                 g_key_file_set_string (keyfile, key, "Name", name ? name : "");
                 g_key_file_set_string (keyfile, key, "Fullname", fullname ? fullname : "");
                 g_key_file_set_string (keyfile, key, "Description", description ? description : "");
@@ -872,7 +895,7 @@ save_chefs (GrRecipeStore *store)
 static void
 save_user (GrRecipeStore *self)
 {
-        g_autoptr(GSettings) settings = g_settings_new ("org.gnome.Recipes");
+        GSettings *settings = gr_settings_get ();
         g_settings_set_string (settings, "user", self->user ? self->user : "");
 }
 
@@ -880,7 +903,7 @@ static void
 load_user (GrRecipeStore *self,
            const char    *dir)
 {
-        g_autoptr(GSettings) settings = g_settings_new ("org.gnome.Recipes");
+        GSettings *settings = gr_settings_get ();
         g_autofree char *user = g_settings_get_string (settings, "user");
 
         if (user[0] == '\0') {
@@ -891,46 +914,289 @@ load_user (GrRecipeStore *self,
         self->user = g_strdup (user);
 }
 
+static char *
+get_data_cache_dir (void)
+{
+        char *cache_dir = NULL;
+
+        cache_dir = g_build_filename (get_user_cache_dir (), "data", NULL);
+        g_mkdir_with_parents (cache_dir, 0755);
+
+        return cache_dir;
+}
+
+static gboolean
+should_try_load (const char *path)
+{
+        g_autoptr(GFile) file = NULL;
+        g_autoptr(GFileInfo) info = NULL;
+        gboolean result = TRUE;
+
+        file = g_file_new_for_path (path);
+        info = g_file_query_info (file,
+                                  G_FILE_ATTRIBUTE_TIME_MODIFIED,
+                                  G_FILE_QUERY_INFO_NONE,
+                                  NULL,
+                                  NULL);
+        if (info) {
+                GTimeVal tv;
+                g_autoptr(GDateTime) now = NULL;
+                g_autoptr(GDateTime) mtime = NULL;
+
+                g_file_info_get_modification_time (info, &tv);
+
+                now = g_date_time_new_now_utc ();
+                mtime = g_date_time_new_from_timeval_utc (&tv);
+
+                result = g_date_time_difference (now, mtime) > G_TIME_SPAN_DAY;
+                g_debug ("Cached file for %s is %s",
+                         path,
+                         result ? "old, trying again" : "new enough");
+        }
+
+        return result;
+}
+
+static void
+set_modified_request (SoupMessage *msg,
+                      const char  *path)
+{
+        g_autoptr(GFile) file = NULL;
+        g_autoptr(GFileInfo) info = NULL;
+        GTimeVal tv;
+        g_autoptr(GDateTime) mtime = NULL;
+        g_autofree char *mod_date = NULL;
+
+        file = g_file_new_for_path (path);
+        info = g_file_query_info (file,
+                                  G_FILE_ATTRIBUTE_TIME_MODIFIED,
+                                  G_FILE_QUERY_INFO_NONE,
+                                  NULL,
+                                  NULL);
+
+        g_file_info_get_modification_time (info, &tv);
+        mtime = g_date_time_new_from_timeval_utc (&tv);
+        mod_date = g_date_time_format (mtime, "%a, %d %b %Y %H:%M:%S %Z");
+        soup_message_headers_append (msg->request_headers, "If-Modified-Since", mod_date);
+}
+
+
+static void
+update_file_timestamp (const char *path)
+{
+        g_autoptr(GFile) file = NULL;
+        g_autoptr(GDateTime) now = NULL;
+        guint64 mtime;
+
+        now = g_date_time_new_now_utc ();
+        mtime = (guint64)g_date_time_to_unix (now);
+
+        file = g_file_new_for_path (path);
+        g_file_set_attribute_uint64 (file, G_FILE_ATTRIBUTE_TIME_MODIFIED, mtime, 0, NULL, NULL);
+        g_debug ("updating timestamp for %s", path);
+}
+
+static void
+empty_store (GrRecipeStore *self)
+{
+        GHashTableIter iter;
+        char *key;
+        GrRecipe *recipe;
+        GrChef *chef;
+
+        g_hash_table_iter_init (&iter, self->recipes);
+        while (g_hash_table_iter_next (&iter, (gpointer *)&key, (gpointer *)&recipe)) {
+                g_hash_table_iter_remove (&iter);
+        }
+
+        g_hash_table_iter_init (&iter, self->chefs);
+        while (g_hash_table_iter_next (&iter, (gpointer *)&key, (gpointer *)&chef)) {
+                g_hash_table_iter_remove (&iter);
+        }
+
+        g_clear_pointer (&self->todays, g_strfreev);
+        g_clear_pointer (&self->picks, g_strfreev);
+        g_clear_pointer (&self->favorites, g_strfreev);
+        g_clear_pointer (&self->export_list, g_strfreev);
+        g_clear_pointer (&self->featured_chefs, g_strfreev);
+        g_clear_pointer (&self->shopping_list, g_variant_dict_unref);
+}
+
+static void
+reload_updates (GrRecipeStore *self)
+{
+        g_autofree char *cache_dir = NULL;
+        const char *user_dir;
+
+        g_debug ("New data obtained, reloading!");
+
+        empty_store (self);
+
+        cache_dir = get_data_cache_dir ();
+        user_dir = get_user_data_dir ();
+
+        load_recipes (self, cache_dir, TRUE);
+        load_chefs (self, cache_dir, TRUE);
+        load_picks (self, cache_dir);
+
+        load_recipes (self, user_dir, FALSE);
+        load_chefs (self, user_dir, FALSE);
+        load_favorites (self);
+        load_export_list (self);
+        load_shopping (self);
+
+        g_signal_emit_by_name (self, "reloaded", 0);
+}
+
+static void
+tar_done (GObject      *source,
+          GAsyncResult *result,
+          gpointer      data)
+{
+        GrRecipeStore *self = data;
+        g_autoptr(GError) error = NULL;
+
+        if (!g_subprocess_wait_finish (G_SUBPROCESS (source), result, &error)) {
+                g_warning ("Failed to run tar: %s", error->message);
+                return;
+        }
+
+        reload_updates (self);
+}
+
+static void
+save_file (SoupSession *session,
+           SoupMessage *msg,
+           gpointer     data)
+{
+        GrRecipeStore *self = data;
+        const char *cache_dir;
+        g_autofree char *filename = NULL;
+        const char *argv[5];
+        g_autofree char *cmdline = NULL;
+        g_autoptr(GSubprocess) subprocess = NULL;
+        g_autoptr(GSubprocessLauncher) launcher = NULL;
+        g_autoptr(GError) error = NULL;
+
+        if (msg->status_code == SOUP_STATUS_CANCELLED || self->session == NULL) {
+                g_debug ("Message cancelled");
+                goto out;
+        }
+
+        cache_dir = get_user_cache_dir ();
+        filename = g_build_filename (cache_dir, "data.tar.gz", NULL);
+
+        if (msg->status_code == SOUP_STATUS_NOT_MODIFIED) {
+                g_autofree char *f = NULL;
+
+                g_debug ("File not modified");
+                f = g_build_filename (cache_dir, "data", "recipes.db", NULL);
+                update_file_timestamp (f);
+        }
+        else if (msg->status_code == SOUP_STATUS_OK) {
+                g_debug ("Saving file to %s", filename);
+                if (!g_file_set_contents (filename, msg->response_body->data, msg->response_body->length, NULL)) {
+                        g_debug ("Saving file to %s failed", filename);
+                        goto out;
+                }
+        }
+        else {
+                g_warning ("Failed to load %s", filename);
+                goto out;
+        }
+
+        argv[0] = "tar";
+        argv[1] = "-xf";
+        argv[2] = "data.tar.gz";
+        argv[3] = "--overwrite";
+        argv[4] = NULL;
+
+        // FIXME use libarchive instead
+        g_debug ("Running %s", cmdline = g_strjoinv (" ", (char **)argv));
+        launcher = g_subprocess_launcher_new (0);
+        g_subprocess_launcher_set_cwd (launcher, cache_dir);
+        subprocess = g_subprocess_launcher_spawnv (launcher, argv, &error);
+        if (!subprocess) {
+                g_warning ("Failed to create command: %s", error->message);
+                goto out;
+        }
+
+        g_subprocess_wait_async (subprocess, NULL, tar_done, self);
+out:
+        g_clear_object (&self->recipes_message);
+}
+
+#define BASE_URL "https://static.gnome.org/recipes/v1"
+
+static char *
+get_file_url (const char *basename)
+{
+        return g_strconcat (BASE_URL, "/", basename, NULL);
+}
+
+static gboolean
+load_updates (gpointer data)
+{
+        GrRecipeStore *self = data;
+        g_autofree char *cache_dir = NULL;
+        g_autofree char *filename = NULL;
+
+        cache_dir = get_data_cache_dir ();
+        filename = g_build_filename (cache_dir, "recipes.db", NULL);
+        if (should_try_load (filename)) {
+                g_autofree char *url;
+                g_autoptr(SoupURI) base_uri = NULL;
+
+                url = get_file_url ("data.tar.gz");
+                base_uri = soup_uri_new (url);
+                self->recipes_message = soup_message_new_from_uri (SOUP_METHOD_GET, base_uri);
+                set_modified_request (self->recipes_message, filename);
+                g_debug ("Load file for data.tar.gz from %s", url);
+                soup_session_queue_message (self->session, g_object_ref (self->recipes_message), save_file, self);
+        }
+
+        g_timeout_add_seconds (24 * 60 * 60, load_updates, data);
+
+        return G_SOURCE_REMOVE;
+}
+
 static void
 gr_recipe_store_init (GrRecipeStore *self)
 {
         const char *data_dir;
         const char *user_dir;
-        g_autofree char *current_dir = NULL;
-        g_autofree char *uninstalled_dir = NULL;
+        g_autofree char *cache_dir = NULL;
 
         self->recipes = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_object_unref);
         self->chefs = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_object_unref);
+        self->session = gr_app_get_soup_session (GR_APP (g_application_get_default ()));
 
         data_dir = get_pkg_data_dir ();
         user_dir = get_user_data_dir ();
-        current_dir = g_get_current_dir ();
-        uninstalled_dir = g_build_filename (current_dir, "data", NULL);
+        cache_dir = get_data_cache_dir ();
 
         load_user (self, user_dir);
 
-        if (g_getenv ("RECIPES_DATA_DIR")) {
-                const char *dir = g_getenv ("RECIPES_DATA_DIR");
-
-                g_info ("Loading data from RECIPES_DATA_DIR: %s", dir);
-
-                load_recipes (self, dir, FALSE);
-                load_chefs (self, dir, FALSE);
-                load_picks (self, dir);
-
-                g_info ("%d recipes loaded", g_hash_table_size (self->recipes));
-                g_info ("%d chefs loaded", g_hash_table_size (self->chefs));
-
-                return;
-        }
+        load_updates (self);
 
         /* First load preinstalled data */
-        if (!load_recipes (self, data_dir, TRUE))
-                load_recipes (self, uninstalled_dir, TRUE);
-        if (!load_chefs (self, data_dir, TRUE))
-                load_chefs (self, uninstalled_dir, TRUE);
-        if (!load_picks (self, data_dir))
-                load_picks (self, uninstalled_dir);
+#if 0
+        if (load_recipes (self, cache_dir, TRUE)) {
+                g_autofree char *locale = NULL;
+
+                locale = g_build_filename (cache_dir, "locale", NULL);
+                bindtextdomain (GETTEXT_PACKAGE "-data", locale);
+
+                load_chefs (self, cache_dir, TRUE);
+                load_picks (self, cache_dir);
+        }
+        else 
+#endif
+{
+                load_recipes (self, data_dir, TRUE);
+                load_chefs (self, data_dir, TRUE);
+                load_picks (self, data_dir);
+        }
 
         /* Now load saved data */
         load_recipes (self, user_dir, FALSE);
@@ -947,6 +1213,7 @@ static guint add_signal;
 static guint remove_signal;
 static guint changed_signal;
 static guint chefs_changed_signal;
+static guint reloaded_signal;
 
 static void
 gr_recipe_store_class_init (GrRecipeStoreClass *klass)
@@ -983,6 +1250,13 @@ gr_recipe_store_class_init (GrRecipeStoreClass *klass)
                                              NULL, NULL,
                                              NULL,
                                              G_TYPE_NONE, 0);
+        reloaded_signal = g_signal_new ("reloaded",
+                                        G_TYPE_FROM_CLASS (object_class),
+                                        G_SIGNAL_RUN_LAST,
+                                        0,
+                                        NULL, NULL,
+                                        NULL,
+                                        G_TYPE_NONE, 0);
 }
 
 GrRecipeStore *
@@ -1178,6 +1452,24 @@ gr_recipe_store_get_contributors (GrRecipeStore *self,
         return (char **)g_hash_table_get_keys_as_array (chefs, length);
 }
 
+char **
+gr_recipe_store_get_all_cuisines (GrRecipeStore *self,
+                                  guint         *length)
+{
+        GHashTableIter iter;
+        GrRecipe *recipe;
+        g_autoptr(GHashTable) cuisines = NULL;
+
+        cuisines = g_hash_table_new (g_str_hash, g_str_equal);
+
+        g_hash_table_iter_init (&iter, self->recipes);
+        while (g_hash_table_iter_next (&iter, NULL, (gpointer *) &recipe)) {
+                if (gr_recipe_get_cuisine (recipe))
+                        g_hash_table_add (cuisines, (gpointer)gr_recipe_get_cuisine (recipe));
+        }
+        return (char **) g_hash_table_get_keys_as_array (cuisines, length);
+}
+
 const char *
 gr_recipe_store_get_user_key (GrRecipeStore *self)
 {
