@@ -51,15 +51,13 @@ load_pixbuf_fit_size (const char *path,
         GdkPixbuf *pixbuf;
         int dest_x, dest_y, dest_width, dest_height;
 
+        original = gdk_pixbuf_new_from_file_at_size (path, width, height, NULL);
+        if (!original)
+                return NULL;
+
         pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB, TRUE, 8, width, height);
         gdk_pixbuf_fill (pixbuf, 0x00000000);
 
-        original = gdk_pixbuf_new_from_file_at_size (path, width, height, NULL);
-        if (!original) {
-                g_warning ("Failed to load image %s", path);
-                return pixbuf;
-        }
-
         if (pad) {
                 dest_width = gdk_pixbuf_get_width (original);
                 dest_height = gdk_pixbuf_get_height (original);
@@ -89,13 +87,8 @@ load_pixbuf_fill_size (const char *path,
         int x, y;
 
         original = gdk_pixbuf_new_from_file_at_scale (path, -1, height, TRUE, NULL);
-        if (!original) {
-                GdkPixbuf *pixbuf;
-                g_warning ("Failed to load image %s", path);
-                pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB, TRUE, 8, width, height);
-                gdk_pixbuf_fill (pixbuf, 0x00000000);
-                return pixbuf;
-        }
+        if (!original)
+                return NULL;
 
         if (gdk_pixbuf_get_width (original) < width) {
                 g_autoptr(GdkPixbuf) pb1 = NULL;
@@ -131,6 +124,22 @@ get_user_data_dir (void)
         return (const char *)dir;
 }
 
+const char *
+get_user_cache_dir (void)
+{
+        static char *dir = NULL;
+
+        if (!dir) {
+                if (in_flatpak_sandbox ())
+                        dir = g_strdup (g_get_user_cache_dir ());
+                else
+                        dir = g_build_filename (g_get_user_cache_dir (), PACKAGE_NAME, NULL);
+                g_mkdir_with_parents (dir, S_IRWXU | S_IRWXG | S_IRWXO);
+        }
+
+        return (const char *)dir;
+}
+
 const char *
 get_pkg_data_dir (void)
 {
@@ -158,68 +167,6 @@ container_remove_all (GtkContainer *container)
         g_list_free (children);
 }
 
-static void
-gr_utils_widget_css_parsing_error_cb (GtkCssProvider *provider,
-                                      GtkCssSection *section,
-                                      GError *error,
-                                      gpointer user_data)
-{
-        g_warning ("CSS parse error %u:%u: %s",
-                   gtk_css_section_get_start_line (section),
-                   gtk_css_section_get_start_position (section),
-                   error->message);
-}
-
-static void
-gr_utils_widget_set_css_internal (GtkWidget *widget,
-                                  const gchar *class_name,
-                                  const gchar *css)
-{
-        GtkStyleContext *context;
-        g_autoptr(GtkCssProvider) provider = NULL;
-
-        g_debug ("using custom CSS %s", css);
-
-        /* set the custom CSS class */
-        context = gtk_widget_get_style_context (widget);
-        gtk_style_context_add_class (context, class_name);
-
-        /* set up custom provider and store on the widget */
-        provider = gtk_css_provider_new ();
-        g_signal_connect (provider, "parsing-error",
-                          G_CALLBACK (gr_utils_widget_css_parsing_error_cb), NULL);
-        gtk_style_context_add_provider_for_screen (gdk_screen_get_default (),
-                                                   GTK_STYLE_PROVIDER (provider),
-                                                   GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
-        gtk_css_provider_load_from_data (provider, css, -1, NULL);
-        g_object_set_data_full (G_OBJECT (widget),
-                                "custom-css-provider",
-                                g_object_ref (provider),
-                                g_object_unref);
-}
-
-void
-gr_utils_widget_set_css_simple (GtkWidget *widget,
-                                const      gchar *css)
-{
-        g_autofree gchar *class_name = NULL;
-        g_autoptr(GString) str = NULL;
-
-        /* remove custom class if NULL */
-        class_name = g_strdup_printf ("themed-widget_%p", widget);
-        if (css == NULL) {
-                GtkStyleContext *context = gtk_widget_get_style_context (widget);
-                gtk_style_context_remove_class (context, class_name);
-                return;
-        }
-        str = g_string_sized_new (1024);
-        g_string_append_printf (str, ".%s {\n", class_name);
-        g_string_append_printf (str, "%s\n", css);
-        g_string_append (str, "}");
-
-        gr_utils_widget_set_css_internal (widget, class_name, str->str);
-}
-
 char *
 date_time_to_string (GDateTime *dt)
 {
@@ -457,7 +404,7 @@ portal_available (GtkWindow  *window,
 
         if (strcmp (interface, "org.freedesktop.portal.FileChooser") == 0)
                 message1 = _("Missing the desktop portal needed to open files from inside a Flatpak sandbox.");
-        else if (strcmp (interface, "org.freedesktop.portal.Print") == 0)
+        else if (strcmp (interface, "org.freedestkop.portal.Print") == 0)
                 message1 = _("Missing the desktop portal needed to print from inside a Flatpak sandbox.");
         else if (strcmp (interface, "org.freedesktop.portal.OpenURI") == 0)
                 message1 = _("Missing the desktop portal needed to open URLs from inside a Flatpak sandbox.");
@@ -716,3 +663,136 @@ remove_image (const char *path)
                 g_print ("Not removing image %s", path);
         }
 }
+
+/* blur code borrowed from libappstream-glib */
+static void
+pixbuf_blur_private (GdkPixbuf *src, GdkPixbuf *dest, gint radius, guchar *div_kernel_size)
+{
+        gint width, height, src_rowstride, dest_rowstride, n_channels;
+        guchar *p_src, *p_dest, *c1, *c2;
+        gint x, y, i, i1, i2, width_minus_1, height_minus_1, radius_plus_1;
+        gint r, g, b, a;
+        guchar *p_dest_row, *p_dest_col;
+
+        width = gdk_pixbuf_get_width (src);
+        height = gdk_pixbuf_get_height (src);
+        n_channels = gdk_pixbuf_get_n_channels (src);
+        radius_plus_1 = radius + 1;
+
+        /* horizontal blur */
+        p_src = gdk_pixbuf_get_pixels (src);
+        p_dest = gdk_pixbuf_get_pixels (dest);
+        src_rowstride = gdk_pixbuf_get_rowstride (src);
+        dest_rowstride = gdk_pixbuf_get_rowstride (dest);
+        width_minus_1 = width - 1;
+        for (y = 0; y < height; y++) {
+
+                /* calc the initial sums of the kernel */
+                r = g = b = a = 0;
+                for (i = -radius; i <= radius; i++) {
+                        c1 = p_src + (CLAMP (i, 0, width_minus_1) * n_channels);
+                        r += c1[0];
+                        g += c1[1];
+                        b += c1[2];
+                }
+
+                p_dest_row = p_dest;
+                for (x = 0; x < width; x++) {
+                        /* set as the mean of the kernel */
+                        p_dest_row[0] = div_kernel_size[r];
+                        p_dest_row[1] = div_kernel_size[g];
+                        p_dest_row[2] = div_kernel_size[b];
+                        p_dest_row += n_channels;
+
+                        /* the pixel to add to the kernel */
+                        i1 = x + radius_plus_1;
+                        if (i1 > width_minus_1)
+                                i1 = width_minus_1;
+                        c1 = p_src + (i1 * n_channels);
+
+                        /* the pixel to remove from the kernel */
+                        i2 = x - radius;
+                        if (i2 < 0)
+                                i2 = 0;
+                        c2 = p_src + (i2 * n_channels);
+
+                        /* calc the new sums of the kernel */
+                        r += c1[0] - c2[0];
+                        g += c1[1] - c2[1];
+                        b += c1[2] - c2[2];
+                }
+
+                p_src += src_rowstride;
+                p_dest += dest_rowstride;
+        }
+
+        /* vertical blur */
+        p_src = gdk_pixbuf_get_pixels (dest);
+        p_dest = gdk_pixbuf_get_pixels (src);
+        src_rowstride = gdk_pixbuf_get_rowstride (dest);
+        dest_rowstride = gdk_pixbuf_get_rowstride (src);
+        height_minus_1 = height - 1;
+        for (x = 0; x < width; x++) {
+
+                /* calc the initial sums of the kernel */
+                r = g = b = a = 0;
+                for (i = -radius; i <= radius; i++) {
+                        c1 = p_src + (CLAMP (i, 0, height_minus_1) * src_rowstride);
+                        r += c1[0];
+                        g += c1[1];
+                        b += c1[2];
+                }
+
+                p_dest_col = p_dest;
+                for (y = 0; y < height; y++) {
+                        /* set as the mean of the kernel */
+
+                        p_dest_col[0] = div_kernel_size[r];
+                        p_dest_col[1] = div_kernel_size[g];
+                        p_dest_col[2] = div_kernel_size[b];
+                        p_dest_col += dest_rowstride;
+
+                        /* the pixel to add to the kernel */
+                        i1 = y + radius_plus_1;
+                        if (i1 > height_minus_1)
+                                i1 = height_minus_1;
+                        c1 = p_src + (i1 * src_rowstride);
+
+                        /* the pixel to remove from the kernel */
+                        i2 = y - radius;
+                        if (i2 < 0)
+                                i2 = 0;
+                        c2 = p_src + (i2 * src_rowstride);
+                        /* calc the new sums of the kernel */
+                        r += c1[0] - c2[0];
+                        g += c1[1] - c2[1];
+                        b += c1[2] - c2[2];
+                }
+
+                p_src += n_channels;
+                p_dest += n_channels;
+        }
+}
+
+void
+pixbuf_blur (GdkPixbuf *src, gint radius, gint iterations)
+{
+        gint kernel_size;
+        gint i;
+        g_autofree guchar *div_kernel_size = NULL;
+        g_autoptr(GdkPixbuf) tmp = NULL;
+
+        tmp = gdk_pixbuf_new (gdk_pixbuf_get_colorspace (src),
+                              gdk_pixbuf_get_has_alpha (src),
+                              gdk_pixbuf_get_bits_per_sample (src),
+                              gdk_pixbuf_get_width (src),
+                              gdk_pixbuf_get_height (src));
+        kernel_size = 2 * radius + 1;
+        div_kernel_size = g_new (guchar, 256 * kernel_size);
+        for (i = 0; i < 256 * kernel_size; i++)
+                div_kernel_size[i] = (guchar) (i / kernel_size);
+
+        while (iterations-- > 0)
+                pixbuf_blur_private (src, tmp, radius, div_kernel_size);
+}
+
