@@ -32,15 +32,16 @@ GdkPixbuf *load_pixbuf_fill_size (const char *path,
                                   int         width,
                                   int         height);
 
+void pixbuf_blur (GdkPixbuf *src,
+                  int        radius,
+                  int        iterations);
+
 const char *get_pkg_data_dir  (void);
 const char *get_user_data_dir (void);
-const char *get_old_user_data_dir (void);
+const char *get_user_cache_dir (void);
 
 void    container_remove_all (GtkContainer *container);
 
-void gr_utils_widget_set_css_simple (GtkWidget  *widget,
-                                     const char *css);
-
 char      * date_time_to_string   (GDateTime *dt);
 GDateTime * date_time_from_string (const char *string);
 char      * format_date_time_difference (GDateTime *end, GDateTime *start);
@@ -57,7 +58,8 @@ void stop_recording (void);
 void record_step (const char *blurb);
 
 gboolean in_flatpak_sandbox (void);
-gboolean portals_available (void);
+gboolean portal_available (GtkWindow  *window,
+                           const char *portal);
 
 void all_headers (GtkListBoxRow *row,
                   GtkListBoxRow *before,
